@@ -51,11 +51,8 @@ class SelfExporter(object):
             if SelfExporter.default_model is not None \
                     and new_target_module is SelfExporter.target_module:
                 warnings.warn("Redefining model! (You can probably ignore this if you are running"
-                              " code interactively)", ModelExistsWarning, stacklevel);
-                # delete previously exported symbols to prevent extra SymbolExistsWarnings
-                for name in [c.name for c in SelfExporter.default_model.all_components()] + ['model']:
-                    if name in SelfExporter.target_globals:
-                        del SelfExporter.target_globals[name]
+                              " code interactively)", ModelExistsWarning, stacklevel)
+                SelfExporter.cleanup()
             SelfExporter.target_module = new_target_module
             SelfExporter.target_globals = caller_frame.f_globals
             SelfExporter.default_model = obj
@@ -84,6 +81,16 @@ class SelfExporter(object):
             warnings.warn("'%s' already defined" % (export_name), SymbolExistsWarning, stacklevel)
         SelfExporter.target_globals[export_name] = obj
 
+    @staticmethod
+    def cleanup():
+        # delete previously exported symbols
+        for name in [c.name for c in SelfExporter.default_model.all_components()] + ['model']:
+            if name in SelfExporter.target_globals:
+                del SelfExporter.target_globals[name]
+        SelfExporter.default_model = None
+        SelfExporter.target_globals = None
+        SelfExporter.target_module = None
+
     @staticmethod
     def rename(obj, new_name):
         """Rename a previously exported symbol"""
@@ -297,6 +304,14 @@ class MonomerPattern(object):
     def __rshift__(self, other):
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
             return RuleExpression(self, other, False)
+        elif other is None:
+            return RuleExpression(self, ReactionPattern([]), False)
+        else:
+            return NotImplemented
+
+    def __rrshift__(self, other):
+        if other is None:
+            return RuleExpression(ReactionPattern([]), self, False)
         else:
             return NotImplemented
 
@@ -403,6 +418,14 @@ class ComplexPattern(object):
     def __rshift__(self, other):
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
             return RuleExpression(self, other, False)
+        elif other is None:
+            return RuleExpression(self, ReactionPattern([]), False)
+        else:
+            return NotImplemented
+
+    def __rrshift__(self, other):
+        if other is None:
+            return RuleExpression(ReactionPattern([]), self, False)
         else:
             return NotImplemented
 
@@ -455,6 +478,14 @@ class ReactionPattern(object):
         """Irreversible reaction"""
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
             return RuleExpression(self, other, False)
+        elif other is None:
+            return RuleExpression(self, ReactionPattern([]), False)
+        else:
+            return NotImplemented
+
+    def __rrshift__(self, other):
+        if other is None:
+            return RuleExpression(ReactionPattern([]), self, False)
         else:
             return NotImplemented
 
@@ -462,11 +493,16 @@ class ReactionPattern(object):
         """Reversible reaction"""
         if isinstance(other, (MonomerPattern, ComplexPattern, ReactionPattern)):
             return RuleExpression(self, other, True)
+        elif other is None:
+            return RuleExpression(self, ReactionPattern([]), True)
         else:
             return NotImplemented
 
     def __repr__(self):
-        return ' + '.join([repr(p) for p in self.complex_patterns])
+        if len(self.complex_patterns):
+            return ' + '.join([repr(p) for p in self.complex_patterns])
+        else:
+            return 'None'
 
 
 
@@ -494,6 +530,10 @@ class RuleExpression(object):
             raise type(e)("Product does not look like a reaction pattern")
         self.is_reversible = is_reversible
 
+    def __repr__(self):
+        operator = '<>' if self.is_reversible else '>>'
+        return '%s %s %s' % (repr(self.reactant_pattern), operator,
+                             repr(self.product_pattern))
 
 
 def as_complex_pattern(v):
@@ -593,6 +633,12 @@ class Rule(Component):
         self.move_connected = move_connected
         # TODO: ensure all numbered sites are referenced exactly twice within each of reactants and products
 
+    def is_synth(self):
+        return len(self.reactant_pattern.complex_patterns) == 0
+
+    def is_deg(self):
+        return len(self.product_pattern.complex_patterns) == 0
+
     def __repr__(self):
         ret = '%s(name=%s, reactants=%s, products=%s, rate_forward=%s' % \
             (self.__class__.__name__, repr(self.name), repr(self.reactant_pattern), repr(self.product_pattern), repr(self.rate_forward))
@@ -625,6 +671,11 @@ class Observable(Component):
         self.species = []
         self.coefficients = []
 
+    def __repr__(self):
+        ret = '%s(%s, %s)' % (self.__class__.__name__, repr(self.name),
+                              repr(self.reaction_pattern))
+        return ret
+
 
 
 class Model(object):
@@ -727,20 +778,94 @@ class Model(object):
             if component in cset:
                 cset.rename(component, new_name)
 
-    def initial(self, complex_pattern, value):
+    def _validate_initial_condition_pattern(self, pattern):
+        """Make sure the initial condition pattern is valid.
+
+        Patterns must be:
+
+
+        Parameters
+        ----------
+        pattern : A Monomer, MonomerPattern, or ComplexPattern
+            To be valid, a pattern must be:
+
+            - Able to be cast as a ComplexPattern
+            - Concrete (all sites and states specified)
+            - Distinct from any existing initial condition pattern
+
+        Returns
+        -------
+        If the provided pattern is valid, returns it as a ComplexPattern object.
+        object.
+        """
+
         try:
-            complex_pattern = as_complex_pattern(complex_pattern)
+            complex_pattern = as_complex_pattern(pattern)
         except InvalidComplexPatternException as e:
-            raise type(e)("Initial condition species does not look like a ComplexPattern")
-        if not isinstance(value, Parameter):
-            raise Exception("Value must be a Parameter")
+            raise type(e)("Initial condition species does not look like a "
+                          "ComplexPattern")
         if not complex_pattern.is_concrete():
             raise Exception("Pattern must be concrete")
-        if any(complex_pattern.is_equivalent_to(other_cp) for other_cp, value in self.initial_conditions):
-            # FIXME until we get proper canonicalization this could produce false negatives
+        if any(complex_pattern.is_equivalent_to(other_cp)
+               for other_cp, value in self.initial_conditions):
+            # FIXME until we get proper canonicalization this could produce
+            # false negatives
             raise Exception("Duplicate initial condition")
+        return complex_pattern
+
+    def initial(self, pattern, value):
+        complex_pattern = self._validate_initial_condition_pattern(pattern)
+        if not isinstance(value, Parameter):
+            raise Exception("Value must be a Parameter")
         self.initial_conditions.append( (complex_pattern, value) )
 
+    def update_initial_condition_pattern(self, before_pattern, after_pattern):
+        """Update the concrete pattern associated with an initial condition.
+
+        Leaves the Parameter object associated with the initial condition
+        unchanged while modifying the pattern associated with that condition.
+        For example this is useful for changing the state of a site on a
+        monomer or complex associated with an initial condition without having
+        to create an independent initial condition, and parameter, associated
+        with that alternative state.
+
+        Parameters
+        ----------
+        before_pattern : ComplexPattern
+            The concrete pattern specifying the (already existing) initial
+            condition. If the model does not contain an initial condition
+            for the pattern, a ValueError is raised.
+        after_pattern : ComplexPattern
+            The concrete pattern specifying the new pattern to use to replace
+            before_pattern.
+        """
+        # Get the initial condition index
+        ic_index_list = [i for i, ic in enumerate(self.initial_conditions)
+                   if ic[0].is_equivalent_to(as_complex_pattern(before_pattern))]
+
+        # If the initial condition to replace is not found, raise an error
+        if not ic_index_list:
+            raise ValueError("No initial condition found for pattern %s" %
+                             before_pattern)
+
+        # If more than one matching initial condition is found, raise an
+        # error (this should never happen, because duplicate initial conditions
+        # are not allowed to be created)
+        assert len(ic_index_list) == 1
+        ic_index = ic_index_list[0]
+
+        # Make sure the new initial condition pattern is valid
+        after_pattern = self._validate_initial_condition_pattern(after_pattern)
+
+        # Since everything checks out, replace the old initial condition
+        # pattern with the new one.  Because initial_conditions are tuples (and
+        # hence immutable), we cannot simply replace the pattern; instead we
+        # must delete the old one and add the new one.
+        # We retain the old parameter object:
+        p = self.initial_conditions[ic_index][1]
+        del self.initial_conditions[ic_index]
+        self.initial_conditions.append( (after_pattern, p) )
+
     def get_species_index(self, complex_pattern):
         # FIXME I don't even want to think about the inefficiency of this, but at least it works
         try:
@@ -748,6 +873,23 @@ class Model(object):
         except StopIteration:
             return None
 
+    def has_synth_deg(self):
+        """Return true if model uses synthesis or degradation reactions."""
+        return any(r.is_synth() or r.is_deg() for r in self.rules)
+
+    def enable_synth_deg(self):
+        """Add components needed to support synthesis and degradation rules."""
+        if self.monomers.get('__source') is None:
+            self.add_component(Monomer('__source', _export=False))
+        if self.monomers.get('__sink') is None:
+            self.add_component(Monomer('__sink', _export=False))
+        if self.parameters.get('__source_0') is None:
+            self.add_component(Parameter('__source_0', 1.0, _export=False))
+
+        source_cp = as_complex_pattern(self.monomers['__source']())
+        if not any(source_cp.is_equivalent_to(other_cp) for other_cp, value in self.initial_conditions):
+            self.initial(source_cp, self.parameters['__source_0'])
+
     def reset_equations(self):
         """Clear out anything generated by bng.generate_equations or the like"""
         self.species = []
