@@ -168,13 +168,14 @@ def odesolve(model, tfinal, envlist, params, useparams=None, tinit = 0.0, ic=Tru
     #print "Integration finished"
 
     #now deal with observables
-    obs_names = [name for name, rp in model.observable_patterns]
-    yobs = numpy.zeros([len(obs_names), nsteps])
+    #obs_names = [name for name, rp in model.observable_patterns]
+    yobs = numpy.zeros([len(model.observables), nsteps])
     
     #sum up the correct entities
-    for i, name in enumerate(obs_names):
-        factors, species = zip(*model.observable_groups[name])
-        yobs[i] = (yout[:, species] * factors).sum(1)
+    for i, obs in enumerate(model.observables):
+        coeffs = obs.coefficients
+        specs  = obs.species
+        yobs[i] = (yout[:, specs] * coeffs).sum(1)
 
     #merge the x and y arrays for easy analysis
     xyobs = numpy.vstack((xout, yobs))
@@ -262,7 +263,7 @@ def compare_data(xparray, simarray, xspairlist, vardata=False):
     #print "OBJOUT(total):", objout
     return numpy.asarray(objout)
 
-def getlog(sobolarr, params, omag=1, useparams=None, usemag=None):
+def getlog(sobolarr, params, omag=1, useparams=[], usemag=None):
     # map a set of sobol pseudo-random numbers to a range for parameter evaluation
     # sobol: sobol number array of the appropriate length
     # params: array of parameters
@@ -270,8 +271,6 @@ def getlog(sobolarr, params, omag=1, useparams=None, usemag=None):
     #
 
     sobprmarr = numpy.zeros_like(sobolarr)
-    if useparams is None:
-        useparams = []
     ub = numpy.zeros(len(params))
     lb = numpy.zeros(len(params))
     # set upper/lower bounds for generic problem
@@ -335,10 +334,10 @@ def genCmtx(sobmtxA, sobmtxB):
     and returns the C matrix for simulations.
     See e.g. Saltelli, Ratto, Andres, Campolongo, Cariboni, Gatelli, Saisana, Tarantola Global Sensitivity Analysis"""
 
-    nparams = sobmtxA.shape[1]
+    nparams = sobmtxA.shape[1] # shape 1 should be the number of params
 
     # allocate the space for the C matrix
-    sobmtxC = numpy.array([sobmtxB]*nparams) # shape 1 should be the number of params
+    sobmtxC = numpy.array([sobmtxB]*nparams) 
 
     # Now we have nparams copies of sobmtxB. replace the i_th column of sobmtxC with the i_th column of sobmtxA
     for i in range(nparams):
@@ -347,8 +346,8 @@ def genCmtx(sobmtxA, sobmtxB):
     return sobmtxC
 
 
-def sobolfxn(model, sobmtxA, sobmtxB, sobmtxC, time, envlist, xpdata, xspairlist, ic=True, norm=True, vardata=False, useparams = None, fileobj=None):
-    ''' Sobolfxn calculates the yA, yB, and yC_i arrays needed for variance-based global sensitivity analysis
+def parmeval(model, sobmtxA, sobmtxB, sobmtxC, time, envlist, xpdata, xspairlist, ic=True, norm=True, vardata=False, useparams = None, fileobj=None):
+    ''' Function parmeval calculates the yA, yB, and yC_i arrays needed for variance-based global sensitivity analysis
     as prescribed by Saltelli and derived from the work by Sobol.
     '''
     # 
@@ -429,36 +428,53 @@ def sobolfxn(model, sobmtxA, sobmtxB, sobmtxC, time, envlist, xpdata, xspairlist
 def getvarsens(yA, yB, yC):
     """Calculate the array of S_i and ST_i for each parameter given yA, yB, yC matrices
     from the multi-sampling runs. Calculate S_i and ST_i as follows:
+    
+    Parameter sensitivity:
+    ----------------------
+            U_j - E^2 
+    S_j = ------------
+               V(y)
+ 
+    U_j = 1/n \sum yA * yC_j
+
+    E^2 = 1/n \sum yA * 1/n \sum yB
+
+    Total effect sensitivity (i.e. non additive part):
+    --------------------------------------------------
+                  U_-j - E^2
+     ST_j = 1 - -------------
+                      V(y)
+
+    U_-j = 1/n \sum yB * yC_j
+
+    E^2 = { 1/n \sum yB * yB }^2
 
 
-           yA.yC_i - f_0^2
-     S_i = ---------------
-            yA.yA - f_0^2
+    In both cases, calculate V(y) from yA and yB
 
-                  yB.yC_i - f_0^2
-     ST_i = 1 -  -----------------
-                   yA.yA - f_0^2
 
     """
-    nparms = yC.shape[0]
-    nobs = yC.shape[-1]
+    nparms = yC.shape[0] # should be the number of parameters
+    nsamples = yC.shape[1] # should be the number of samples from the original matrix
+    nobs = yC.shape[-1]    # the number of observables (this is linked to BNG usage, generalize?)
 
-    # first get f_0^2, it only depends on yA. Notice this is an array of size n_observables
-    f02 = yA.mean(axis=0)
-    f02 = numpy.multiply(f02, f02)
+    #first get V(y) from yA and yB
 
-    # now get the denominator term for yA
-    yAd = numpy.multiply(yA, yA)
-    yAd = yAd.mean(axis=0)
+    varyA = numpy.var(yA, axis=0, ddof=1)
+    varyB = numpy.var(yB, axis=0, ddof=1)
+
+    # now get the E^2 values for the S and ST calculations
+    E_s  = numpy.average((yA * yB), axis=0)
+    E_st = numpy.average(yB, axis=0) ** 2
 
     #allocate the S_i and ST_i arrays
-    Sens  =  numpy.zeros((nparms,nobs))
+    Sens = numpy.zeros((nparms,nobs))
     SensT = numpy.zeros((nparms,nobs))
 
-    
+    # now get the U_j and U_-j values and store them 
     for i in range(nparms):
-        Sens[i] = ((yA * yC[i]).mean(axis=0) - f02)/(yAd - f02)
-        SensT[i] = 1.0 - ((yB * yC[i]).mean(axis=0) - f02)/(yAd - f02)
+        Sens[i]  =        (((yA * yC[i]).sum(axis=0)/(nsamples-1.)) - E_s ) / varyA
+        SensT[i] = 1.0 - ((((yB * yC[i]).sum(axis=0)/(nsamples-1.)) - E_st) / varyB)
 
     return Sens, SensT
         
